@@ -26,17 +26,14 @@
 #include <magic.h>
 #endif
 
+#include <libxfce4util/libxfce4util.h>
 
 
-static guint rstto_thumbnail_size[] =
+enum
 {
-    THUMBNAIL_SIZE_VERY_SMALL_SIZE,
-    THUMBNAIL_SIZE_SMALLER_SIZE,
-    THUMBNAIL_SIZE_SMALL_SIZE,
-    THUMBNAIL_SIZE_NORMAL_SIZE,
-    THUMBNAIL_SIZE_LARGE_SIZE,
-    THUMBNAIL_SIZE_LARGER_SIZE,
-    THUMBNAIL_SIZE_VERY_LARGE_SIZE
+    PIXBUF_STATE_NONE,
+    PIXBUF_STATE_PREVIOUS,
+    PIXBUF_STATE_UPDATED,
 };
 
 enum
@@ -66,8 +63,10 @@ struct _RsttoFilePrivate
     gchar *path;
     gchar *collate_key;
 
-    gchar *thumbnail_path;
-    GdkPixbuf *thumbnails[THUMBNAIL_SIZE_COUNT];
+    RsttoThumbnailState thumbnail_states[RSTTO_THUMBNAIL_FLAVOR_COUNT];
+    gchar *thumbnail_paths[RSTTO_THUMBNAIL_FLAVOR_COUNT];
+    GdkPixbuf *pixbufs[RSTTO_THUMBNAIL_SIZE_COUNT];
+    gint pixbuf_states[RSTTO_THUMBNAIL_FLAVOR_COUNT][RSTTO_THUMBNAIL_SIZE_COUNT];
 
     ExifData *exif_data;
     RsttoImageOrientation orientation;
@@ -89,6 +88,8 @@ rstto_file_init (RsttoFile *r_file)
     r_file->priv->orientation = RSTTO_IMAGE_ORIENT_NOT_DETERMINED;
     r_file->priv->scale = RSTTO_SCALE_NONE;
     r_file->priv->auto_scale = RSTTO_SCALE_NONE;
+    for (gint n = 0; n < RSTTO_THUMBNAIL_FLAVOR_COUNT; n++)
+        r_file->priv->thumbnail_states[n] = RSTTO_THUMBNAIL_STATE_UNPROCESSED;
 }
 
 
@@ -142,11 +143,6 @@ rstto_file_finalize (GObject *object)
         g_free (r_file->priv->path);
         r_file->priv->path = NULL;
     }
-    if (r_file->priv->thumbnail_path)
-    {
-        g_free (r_file->priv->thumbnail_path);
-        r_file->priv->thumbnail_path = NULL;
-    }
     if (r_file->priv->uri)
     {
         g_free (r_file->priv->uri);
@@ -163,12 +159,21 @@ rstto_file_finalize (GObject *object)
         r_file->priv->exif_data = NULL;
     }
 
-    for (i = 0; i < THUMBNAIL_SIZE_COUNT; ++i)
+    for (i = 0; i < RSTTO_THUMBNAIL_FLAVOR_COUNT; ++i)
     {
-        if (r_file->priv->thumbnails[i])
+        if (r_file->priv->thumbnail_paths[i])
         {
-            g_object_unref (r_file->priv->thumbnails[i]);
-            r_file->priv->thumbnails[i] = NULL;
+            g_free (r_file->priv->thumbnail_paths[i]);
+            r_file->priv->thumbnail_paths[i] = NULL;
+        }
+    }
+
+    for (i = 0; i < RSTTO_THUMBNAIL_SIZE_COUNT; ++i)
+    {
+        if (r_file->priv->pixbufs[i])
+        {
+            g_object_unref (r_file->priv->pixbufs[i]);
+            r_file->priv->pixbufs[i] = NULL;
         }
     }
 
@@ -277,12 +282,6 @@ rstto_file_get_file (RsttoFile *r_file)
     return r_file->priv->file;
 }
 
-gboolean
-rstto_file_equal (RsttoFile *r_file_a, RsttoFile *r_file_b)
-{
-    return r_file_a == r_file_b;
-}
-
 const gchar *
 rstto_file_get_display_name (RsttoFile *r_file)
 {
@@ -502,95 +501,164 @@ rstto_file_has_exif (RsttoFile *r_file)
     return TRUE;
 }
 
-const gchar *
-rstto_file_get_thumbnail_path (RsttoFile *r_file)
+void
+rstto_file_set_thumbnail_state (RsttoFile *r_file,
+                                RsttoThumbnailFlavor flavor,
+                                RsttoThumbnailState state)
 {
-    const gchar *uri;
-    gchar *checksum, *filename, *cache_dir;
+    r_file->priv->thumbnail_states[flavor] = state;
+}
 
-    if (NULL == r_file->priv->thumbnail_path)
+static const gchar *
+rstto_file_get_thumbnail_path (RsttoFile *r_file,
+                               RsttoThumbnailFlavor flavor)
+{
+    const gchar *uri, *flavor_name;
+    gchar *path, *checksum, *filename;
+
+    if (r_file->priv->thumbnail_paths[flavor] == NULL)
     {
+        flavor_name = rstto_util_get_thumbnail_flavor_name (flavor);
         uri = rstto_file_get_uri (r_file);
         checksum = g_compute_checksum_for_string (G_CHECKSUM_MD5, uri, strlen (uri));
         filename = g_strconcat (checksum, ".png", NULL);
 
         /* build and check if the thumbnail is in the new location */
-        if (g_file_test ("/.flatpak-info", G_FILE_TEST_EXISTS))
+        path = g_build_path ("/", g_get_user_cache_dir (), "thumbnails",
+                             flavor_name, filename, NULL);
+        if (! g_file_test (path, G_FILE_TEST_EXISTS))
         {
-            cache_dir = g_strdup (g_getenv ("HOST_XDG_CACHE_HOME"));
-            if (cache_dir == NULL)
-                cache_dir =  g_build_path ("/", g_get_home_dir (), ".cache", NULL);
-        }
-        else
-            cache_dir = g_strdup (g_get_user_cache_dir ());
-
-        r_file->priv->thumbnail_path = g_build_path ("/", cache_dir, "thumbnails", "normal", filename, NULL);
-
-        if (!g_file_test (r_file->priv->thumbnail_path, G_FILE_TEST_EXISTS))
-        {
-            /* Fallback to old version */
-            g_free (r_file->priv->thumbnail_path);
-
-            r_file->priv->thumbnail_path = g_build_path ("/", g_get_home_dir (), ".thumbnails", "normal", filename, NULL);
-            if (!g_file_test (r_file->priv->thumbnail_path, G_FILE_TEST_EXISTS))
+            /* fallback to old version */
+            g_free (path);
+            path = g_build_path ("/", g_get_home_dir (), ".thumbnails",
+                                 flavor_name, filename, NULL);
+            if (! g_file_test (path, G_FILE_TEST_EXISTS))
             {
-                /* Thumbnail doesn't exist in either spot */
-                g_free (r_file->priv->thumbnail_path);
-                r_file->priv->thumbnail_path = NULL;
+#if LIBXFCE4UTIL_CHECK_VERSION (4, 17, 1)
+                /* fallback to shared repository */
+                g_free (path);
+                path = xfce_create_shared_thumbnail_path (uri, flavor_name);
+                if (path == NULL || ! g_file_test (path, G_FILE_TEST_EXISTS))
+#endif
+                {
+                    /* thumbnail doesn't exist in either spot */
+                    g_free (path);
+                    path = NULL;
+                }
             }
         }
 
         g_free (checksum);
         g_free (filename);
-        g_free (cache_dir);
+
+        r_file->priv->thumbnail_paths[flavor] = path;
     }
 
-    return r_file->priv->thumbnail_path;
+    return r_file->priv->thumbnail_paths[flavor];
 }
 
 const GdkPixbuf *
-rstto_file_get_thumbnail (
-        RsttoFile *r_file,
-        RsttoThumbnailSize size)
+rstto_file_get_thumbnail (RsttoFile *r_file,
+                          RsttoThumbnailSize size)
 {
-    const gchar *thumbnail_path;
     RsttoThumbnailer *thumbnailer;
+    RsttoThumbnailFlavor flavor;
+    GError *error = NULL;
+    const gchar *thumbnail_path;
+    guint n_pixels;
 
-    if (r_file->priv->thumbnails[size])
-        return r_file->priv->thumbnails[size];
-
-    thumbnail_path = rstto_file_get_thumbnail_path (r_file);
-
-    thumbnailer = rstto_thumbnailer_new ();
-    rstto_thumbnailer_queue_file (thumbnailer, r_file);
-
-    if (NULL == thumbnail_path)
+    flavor = rstto_util_get_thumbnail_flavor (size);
+    switch (r_file->priv->thumbnail_states[flavor])
     {
-        /* No thumbnail to return at this time */
-        g_object_unref (thumbnailer);
+        case RSTTO_THUMBNAIL_STATE_PROCESSED:
+            if (r_file->priv->pixbuf_states[flavor][size] == PIXBUF_STATE_UPDATED)
+                return r_file->priv->pixbufs[size];
+            else
+                r_file->priv->pixbuf_states[flavor][size] = PIXBUF_STATE_UPDATED;
+
+            break;
+
+        case RSTTO_THUMBNAIL_STATE_IN_PROCESS:
+            if (r_file->priv->pixbuf_states[flavor][size] == PIXBUF_STATE_PREVIOUS)
+                return r_file->priv->pixbufs[size];
+            else
+                r_file->priv->pixbuf_states[flavor][size] = PIXBUF_STATE_PREVIOUS;
+
+            break;
+
+        case RSTTO_THUMBNAIL_STATE_UNPROCESSED:
+            thumbnailer = rstto_thumbnailer_new ();
+            rstto_thumbnailer_queue_file (thumbnailer, flavor, r_file);
+            g_object_unref (thumbnailer);
+
+            r_file->priv->thumbnail_states[flavor] = RSTTO_THUMBNAIL_STATE_IN_PROCESS;
+            for (gint i = 0; i < RSTTO_THUMBNAIL_SIZE_COUNT; i++)
+                r_file->priv->pixbuf_states[flavor][i] = PIXBUF_STATE_NONE;
+
+            r_file->priv->pixbuf_states[flavor][size] = PIXBUF_STATE_PREVIOUS;
+            break;
+
+        case RSTTO_THUMBNAIL_STATE_ERROR:
+            return r_file->priv->pixbufs[size];
+            break;
+
+        default:
+            g_warn_if_reached ();
+            return NULL;
+    }
+
+    thumbnail_path = rstto_file_get_thumbnail_path (r_file, flavor);
+    if (thumbnail_path == NULL)
+    {
+        if (r_file->priv->thumbnail_states[flavor] == RSTTO_THUMBNAIL_STATE_PROCESSED)
+            g_warn_if_reached ();
+
         return NULL;
     }
 
-    /* FIXME:
-     *
-     * The thumbnail should be updated on the "ready" signal
-     * of the thumbnailer, to account for changed thumbnails
-     * aswell as missing ones.
-     */
-    r_file->priv->thumbnails[size] = gdk_pixbuf_new_from_file_at_scale (
-            thumbnail_path,
-            rstto_thumbnail_size[size],
-            rstto_thumbnail_size[size],
-            TRUE,
-            NULL);
+    n_pixels = rstto_util_get_thumbnail_n_pixels (size);
+    r_file->priv->pixbufs[size] =
+        gdk_pixbuf_new_from_file_at_scale (thumbnail_path, n_pixels, n_pixels, TRUE, &error);
+    if (error != NULL)
+    {
+        g_warning ("Thumbnail pixbuf generation failed for '%s': %s",
+                   r_file->priv->uri, error->message);
+        g_error_free (error);
+    }
 
-    g_object_unref (thumbnailer);
-
-    return r_file->priv->thumbnails[size];
+    return r_file->priv->pixbufs[size];
 }
 
 void
 rstto_file_changed (RsttoFile *r_file)
 {
-    g_signal_emit (r_file, rstto_file_signals[RSTTO_FILE_SIGNAL_CHANGED], 0, NULL);
+    if (r_file->priv->content_type != NULL)
+    {
+        g_free (r_file->priv->content_type);
+        r_file->priv->content_type = NULL;
+    }
+
+    if (r_file->priv->exif_data != NULL)
+    {
+        exif_data_free (r_file->priv->exif_data);
+        r_file->priv->exif_data = NULL;
+    }
+
+    r_file->priv->final_content_type = FALSE;
+    r_file->priv->orientation = RSTTO_IMAGE_ORIENT_NOT_DETERMINED;
+    r_file->priv->scale = RSTTO_SCALE_NONE;
+    for (gint n = 0; n < RSTTO_THUMBNAIL_FLAVOR_COUNT; n++)
+        r_file->priv->thumbnail_states[n] = RSTTO_THUMBNAIL_STATE_UNPROCESSED;
+
+    if (rstto_file_is_valid (r_file))
+    {
+        /* this will send a request to the thumbnailer, which will trigger the necessary
+         * updates with its "ready" signal */
+        rstto_file_get_thumbnail (r_file,
+                                  rstto_util_get_thumbnail_size (RSTTO_THUMBNAIL_FLAVOR_NORMAL));
+
+        g_signal_emit (r_file, rstto_file_signals[RSTTO_FILE_SIGNAL_CHANGED], 0, NULL);
+    }
+    else
+        rstto_image_list_remove_file (rstto_main_window_get_app_image_list (), r_file);
 }
