@@ -35,7 +35,6 @@ enum
 {
     PROP_0,
     PROP_ORIENTATION,
-    PROP_FILE_COLUMN,
     PROP_MODEL,
     PROP_ACTIVE,
     PROP_SHOW_TEXT,
@@ -50,16 +49,10 @@ enum
 
 static guint icon_bar_signals[LAST_SIGNAL];
 
-static GdkPixbuf *thumbnail_missing = NULL;
-
 typedef struct _RsttoIconBarItem RsttoIconBarItem;
 
 
 
-#define RSTTO_ICON_BAR_VALID_MODEL_AND_COLUMNS(obj) \
-            ((obj)->priv->model != NULL && \
-            (obj)->priv->file_column != -1)
-
 static void
 rstto_icon_bar_finalize (GObject *object);
 static void
@@ -121,8 +114,6 @@ rstto_icon_bar_size_request (GtkWidget *widget,
 static gboolean
 rstto_icon_bar_scroll (GtkWidget *widget,
                        GdkEventScroll *event);
-static void
-rstto_icon_bar_invalidate (RsttoIconBar *icon_bar);
 
 static RsttoIconBarItem *
 rstto_icon_bar_get_item_at_pos (RsttoIconBar *icon_bar,
@@ -136,9 +127,6 @@ rstto_icon_bar_paint_item (RsttoIconBar *icon_bar,
                            RsttoIconBarItem *item,
                            cairo_t *cr);
 static void
-rstto_icon_bar_calculate_item_size (RsttoIconBar *icon_bar,
-                                    RsttoIconBarItem *item);
-static void
 cb_rstto_thumbnail_size_changed (GObject *settings,
                                  GParamSpec *pspec,
                                  gpointer user_data);
@@ -154,8 +142,6 @@ rstto_icon_bar_item_new (void);
 static void
 rstto_icon_bar_item_free (gpointer item);
 static void
-rstto_icon_bar_item_invalidate (RsttoIconBarItem *item);
-static void
 rstto_icon_bar_build_items (RsttoIconBar *icon_bar);
 static void
 rstto_icon_bar_row_changed (GtkTreeModel *model,
@@ -190,7 +176,6 @@ struct _RsttoIconBarItem
 {
     GtkTreeIter iter;
     gint index;
-    gint size;
 };
 
 struct _RsttoIconBarPrivate
@@ -200,7 +185,6 @@ struct _RsttoIconBarPrivate
 
     gint            width;
     gint            height;
-    gint            file_column;
 
     RsttoIconBarItem *active_item;
     RsttoIconBarItem *single_click_item;
@@ -208,6 +192,7 @@ struct _RsttoIconBarPrivate
 
     GList          *items;
     gint            item_size;
+    gint            n_visible_items;
     gint            device_scale;
 
     GtkAdjustment  *hadjustment;
@@ -217,13 +202,12 @@ struct _RsttoIconBarPrivate
     RsttoThumbnailer *thumbnailer;
 
     RsttoThumbnailSize thumbnail_size;
+    GdkPixbuf *thumbnail_missing;
 
     GtkOrientation  orientation;
 
     GtkTreeModel   *model;
 
-    PangoLayout    *layout;
-
     gboolean        show_text;
 };
 
@@ -272,22 +256,6 @@ rstto_icon_bar_class_init (RsttoIconBarClass *klass)
                 GTK_ORIENTATION_VERTICAL,
                 G_PARAM_READWRITE));
 
-    /**
-     * RsttoIconBar:file-column:
-     *
-     * The ::file-column property contains the number of the model column
-     * containing the files which are displayed. The text column must be
-     * of type #RSTTO_TYPE_FILE. If this property is set to -1, no
-     * details are displayed.
-     **/
-    g_object_class_install_property (gobject_class,
-            PROP_FILE_COLUMN,
-            g_param_spec_int ("file-column",
-                _("File column"),
-                _("Model column used to retrieve the file from"),
-                -1, G_MAXINT, -1,
-                G_PARAM_READWRITE));
-
     /**
      * RsttoIconBar:model:
      *
@@ -428,8 +396,11 @@ rstto_icon_bar_init (RsttoIconBar *icon_bar)
 {
     icon_bar->priv = rstto_icon_bar_get_instance_private (icon_bar);
 
+    /* if the icon bar is hidden, this applies to the window icon, or other atomic
+     * thumbnail request */
+    icon_bar->priv->n_visible_items = 1;
+
     icon_bar->priv->orientation = GTK_ORIENTATION_VERTICAL;
-    icon_bar->priv->file_column = -1;
     icon_bar->priv->show_text = TRUE;
     icon_bar->priv->settings = rstto_settings_new ();
     icon_bar->priv->thumbnailer = rstto_thumbnailer_new ();
@@ -440,13 +411,6 @@ rstto_icon_bar_init (RsttoIconBar *icon_bar)
 
     rstto_icon_bar_update_missing_icon (icon_bar);
 
-    icon_bar->priv->layout = gtk_widget_create_pango_layout (GTK_WIDGET (icon_bar), NULL);
-    pango_layout_set_width (icon_bar->priv->layout, -1);
-    pango_layout_set_wrap (icon_bar->priv->layout,
-            PANGO_WRAP_WORD_CHAR);
-    pango_layout_set_ellipsize (icon_bar->priv->layout,
-            PANGO_ELLIPSIZE_END);
-
     gtk_widget_set_can_focus (GTK_WIDGET (icon_bar), FALSE);
 
     g_signal_connect (icon_bar->priv->settings, "notify::thumbnail-size",
@@ -479,9 +443,10 @@ rstto_icon_bar_finalize (GObject *object)
 {
     RsttoIconBar *icon_bar = RSTTO_ICON_BAR (object);
 
-    g_object_unref (icon_bar->priv->layout);
     g_object_unref (icon_bar->priv->settings);
     g_object_unref (icon_bar->priv->thumbnailer);
+    if (icon_bar->priv->thumbnail_missing != NULL)
+        g_object_unref (icon_bar->priv->thumbnail_missing);
 
     G_OBJECT_CLASS (rstto_icon_bar_parent_class)->finalize (object);
 }
@@ -503,10 +468,6 @@ rstto_icon_bar_get_property (
             g_value_set_enum (value, icon_bar->priv->orientation);
             break;
 
-        case PROP_FILE_COLUMN:
-            g_value_set_int (value, icon_bar->priv->file_column);
-            break;
-
         case PROP_MODEL:
             g_value_set_object (value, icon_bar->priv->model);
             break;
@@ -547,10 +508,6 @@ rstto_icon_bar_set_property (
             rstto_icon_bar_set_orientation (icon_bar, g_value_get_enum (value));
             break;
 
-        case PROP_FILE_COLUMN:
-            rstto_icon_bar_set_file_column (icon_bar, g_value_get_int (value));
-            break;
-
         case PROP_MODEL:
             rstto_icon_bar_set_model (icon_bar, g_value_get_object (value));
             break;
@@ -594,14 +551,15 @@ rstto_icon_bar_set_device_scale (GObject *window,
                                  gpointer data)
 {
     RsttoIconBar *icon_bar = data;
+    GtkWidget *widget = data;
     GtkRequisition requisition;
 
     g_object_get (window, "device-scale", &icon_bar->priv->device_scale, NULL);
 
     /* do not scale the thumbnails with the rest of the window */
-    rstto_icon_bar_size_request (GTK_WIDGET (icon_bar), &requisition);
+    rstto_icon_bar_size_request (widget, &requisition);
 
-    rstto_icon_bar_invalidate (icon_bar);
+    gtk_widget_queue_resize (widget);
 }
 
 
@@ -681,37 +639,35 @@ rstto_icon_bar_size_request (
         GtkWidget      *widget,
         GtkRequisition *requisition)
 {
-    RsttoIconBarItem *item;
     RsttoIconBar *icon_bar = RSTTO_ICON_BAR (widget);
-    GList *lp;
-    gint n, max_size = 0;
+    gint focus_width, focus_pad, n_items;
 
-    if (! RSTTO_ICON_BAR_VALID_MODEL_AND_COLUMNS (icon_bar)
-        || icon_bar->priv->items == NULL)
+    if (icon_bar->priv->model == NULL || icon_bar->priv->items == NULL)
     {
         icon_bar->priv->width = requisition->width = 0;
         icon_bar->priv->height = requisition->height = 0;
+        icon_bar->priv->item_size = 0;
+
         return;
     }
 
-    /* calculate max item size */
-    for (n = 0, lp = icon_bar->priv->items; lp != NULL; ++n, lp = lp->next)
-    {
-        item = lp->data;
-        rstto_icon_bar_calculate_item_size (icon_bar, item);
-        max_size = MAX (max_size, item->size);
-    }
+    gtk_widget_style_get (widget, "focus-line-width", &focus_width,
+                          "focus-padding", &focus_pad, NULL);
 
-    icon_bar->priv->item_size = max_size;
+    /* calculate item size: there is a focus padding both inside and outside the item */
+    icon_bar->priv->item_size = (rstto_util_get_thumbnail_n_pixels (icon_bar->priv->thumbnail_size)
+                                    + 2 * (focus_width + 2 * focus_pad))
+                                / (gdouble) icon_bar->priv->device_scale;
 
+    n_items = rstto_image_list_get_n_images (RSTTO_IMAGE_LIST (icon_bar->priv->model));
     if (icon_bar->priv->orientation == GTK_ORIENTATION_VERTICAL)
     {
         icon_bar->priv->width = requisition->width = icon_bar->priv->item_size;
-        icon_bar->priv->height = requisition->height = n * icon_bar->priv->item_size;
+        icon_bar->priv->height = requisition->height = n_items * icon_bar->priv->item_size;
     }
     else
     {
-        icon_bar->priv->width = requisition->width = n * icon_bar->priv->item_size;
+        icon_bar->priv->width = requisition->width = n_items * icon_bar->priv->item_size;
         icon_bar->priv->height = requisition->height = icon_bar->priv->item_size;
     }
 }
@@ -760,26 +716,33 @@ rstto_icon_bar_draw (GtkWidget *widget,
                      cairo_t *cr)
 {
     RsttoIconBar *icon_bar = RSTTO_ICON_BAR (widget);
+    GtkAdjustment *adjustment;
     GdkRectangle rect;
     GList *lp;
-    gint offset, n_items, n;
+    gint first_visible, last_visible, offset, n_items, n;
 
     rstto_util_paint_background_color (widget, icon_bar->priv->settings, cr);
 
     gdk_cairo_get_clip_rectangle (cr, &rect);
     if (icon_bar->priv->orientation == GTK_ORIENTATION_VERTICAL)
     {
+        adjustment = icon_bar->priv->vadjustment;
         offset = rect.y / icon_bar->priv->item_size;
         n_items = rect.height / icon_bar->priv->item_size + 2;
     }
     else
     {
+        adjustment = icon_bar->priv->hadjustment;
         offset = rect.x / icon_bar->priv->item_size;
         n_items = rect.width / icon_bar->priv->item_size + 2;
     }
 
-    /* restrict thumbnailer queue size */
-    rstto_thumbnailer_set_n_visible_items (icon_bar->priv->thumbnailer, n_items);
+    /* the right limits for the visible area are given by the adjustemts, and this is
+     * what we use to set the thumbnailer queue size */
+    first_visible = gtk_adjustment_get_value (adjustment) / icon_bar->priv->item_size;
+    last_visible = first_visible + icon_bar->priv->n_visible_items - 1;
+    offset = MIN (MAX (offset, first_visible), last_visible);
+    n_items = MIN (n_items, last_visible - offset + 1);
 
     /* skip items before the drawing area */
     for (lp = icon_bar->priv->items, n = 0; lp != NULL && n < offset; lp = lp->next, n++);
@@ -944,17 +907,6 @@ rstto_icon_bar_button_release (
 
 
 
-static void
-rstto_icon_bar_invalidate (RsttoIconBar *icon_bar)
-{
-    GList *lp;
-
-    for (lp = icon_bar->priv->items; lp != NULL; lp = lp->next)
-        rstto_icon_bar_item_invalidate (lp->data);
-
-    gtk_widget_queue_resize (GTK_WIDGET (icon_bar));
-}
-
 static RsttoIconBarItem *
 rstto_icon_bar_get_item_at_pos (
         RsttoIconBar *icon_bar,
@@ -1011,7 +963,6 @@ rstto_icon_bar_paint_item (
         cairo_t          *cr)
 {
     RsttoFile       *file;
-    GtkTreeIter      iter;
     const GdkPixbuf *pixbuf = NULL;
     GdkRGBA         *border_color, *fill_color;
     GdkRGBA          tmp_color;
@@ -1019,7 +970,7 @@ rstto_icon_bar_paint_item (
     gint             x, y, ifocus_width, ifocus_pad;
     gint             pixbuf_width = 0, pixbuf_height = 0;
 
-    if (!RSTTO_ICON_BAR_VALID_MODEL_AND_COLUMNS (icon_bar))
+    if (icon_bar->priv->model == NULL)
         return;
 
     gtk_widget_style_get (GTK_WIDGET (icon_bar),
@@ -1029,19 +980,11 @@ rstto_icon_bar_paint_item (
     focus_width = (gdouble) ifocus_width / icon_bar->priv->device_scale;
     focus_pad = (gdouble) ifocus_pad / icon_bar->priv->device_scale;
 
-    iter = item->iter;
-    gtk_tree_model_get (icon_bar->priv->model, &iter,
-            icon_bar->priv->file_column, &file,
-            -1);
-
+    gtk_tree_model_get (icon_bar->priv->model, &item->iter, 0, &file, -1);
     pixbuf = rstto_file_get_thumbnail (file, icon_bar->priv->thumbnail_size);
 
-    g_object_unref (file);
-
     if (NULL == pixbuf)
-    {
-        pixbuf = thumbnail_missing;
-    }
+        pixbuf = icon_bar->priv->thumbnail_missing;
 
     if (pixbuf)
     {
@@ -1148,63 +1091,10 @@ rstto_icon_bar_paint_item (
 
 
 
-static void
-rstto_icon_bar_calculate_item_size (
-        RsttoIconBar     *icon_bar,
-        RsttoIconBarItem *item)
-{
-    gint focus_width, focus_pad, size;
-
-    if (G_LIKELY (item->size != -1))
-        return;
-
-    gtk_widget_style_get (GTK_WIDGET (icon_bar),
-            "focus-line-width", &focus_width,
-            "focus-padding", &focus_pad,
-            NULL);
-
-    switch (icon_bar->priv->thumbnail_size)
-    {
-        case THUMBNAIL_SIZE_VERY_SMALL:
-            size = THUMBNAIL_SIZE_VERY_SMALL_SIZE;
-            break;
-        case THUMBNAIL_SIZE_SMALLER:
-            size = THUMBNAIL_SIZE_SMALLER_SIZE;
-            break;
-        case THUMBNAIL_SIZE_SMALL:
-            size = THUMBNAIL_SIZE_SMALL_SIZE;
-            break;
-        case THUMBNAIL_SIZE_NORMAL:
-            size = THUMBNAIL_SIZE_NORMAL_SIZE;
-            break;
-        case THUMBNAIL_SIZE_LARGE:
-            size = THUMBNAIL_SIZE_LARGE_SIZE;
-            break;
-        case THUMBNAIL_SIZE_LARGER:
-            size = THUMBNAIL_SIZE_LARGER_SIZE;
-            break;
-        case THUMBNAIL_SIZE_VERY_LARGE:
-            size = THUMBNAIL_SIZE_VERY_LARGE_SIZE;
-            break;
-        default:
-            size = THUMBNAIL_SIZE_NORMAL_SIZE;
-            break;
-    }
-
-    /* there is a focus padding both inside and outside the item */
-    item->size = (size + 2 * (focus_width + 2 * focus_pad))
-                 / (gdouble) icon_bar->priv->device_scale;
-}
-
 static RsttoIconBarItem *
 rstto_icon_bar_item_new (void)
 {
-    RsttoIconBarItem *item;
-
-    item = g_slice_new0 (RsttoIconBarItem);
-    item->size = -1;
-
-    return item;
+    return g_slice_new0 (RsttoIconBarItem);
 }
 
 
@@ -1217,14 +1107,6 @@ rstto_icon_bar_item_free (gpointer item)
 
 
 
-static void
-rstto_icon_bar_item_invalidate (RsttoIconBarItem *item)
-{
-    item->size = -1;
-}
-
-
-
 static void
 rstto_icon_bar_build_items (RsttoIconBar *icon_bar)
 {
@@ -1269,8 +1151,7 @@ rstto_icon_bar_row_changed (
     if (item == NULL)
         return;
 
-    rstto_icon_bar_item_invalidate (item);
-    gtk_widget_queue_resize (GTK_WIDGET (icon_bar));
+    rstto_icon_bar_queue_draw_item (icon_bar, item);
 }
 
 
@@ -1282,20 +1163,16 @@ rstto_icon_bar_row_inserted (
         GtkTreeIter  *iter,
         RsttoIconBar *icon_bar)
 {
-    RsttoIconBarItem  *item;
-    GList           *lp;
-    gint             idx;
+    RsttoIconBarItem *item;
+    GList *lp;
 
-    idx = gtk_tree_path_get_indices (path)[0];
     item = rstto_icon_bar_item_new ();
+    item->iter = *iter;
+    item->index = gtk_tree_path_get_indices (path)[0];
 
-    if ((gtk_tree_model_get_flags (icon_bar->priv->model) & GTK_TREE_MODEL_ITERS_PERSIST) != 0)
-        item->iter = *iter;
-    item->index = idx;
+    icon_bar->priv->items = g_list_insert (icon_bar->priv->items, item, item->index);
 
-    icon_bar->priv->items = g_list_insert (icon_bar->priv->items, item, idx);
-
-    for (lp = g_list_nth (icon_bar->priv->items, idx + 1); lp != NULL; lp = lp->next)
+    for (lp = g_list_nth (icon_bar->priv->items, item->index + 1); lp != NULL; lp = lp->next)
     {
         item = lp->data;
         item->index++;
@@ -1513,25 +1390,12 @@ rstto_icon_bar_set_model (
         RsttoIconBar *icon_bar,
         GtkTreeModel *model)
 {
-    GType file_column_type;
-
     g_return_if_fail (RSTTO_IS_ICON_BAR (icon_bar));
     g_return_if_fail (GTK_IS_TREE_MODEL (model) || model == NULL);
 
     if (G_UNLIKELY (model == icon_bar->priv->model))
         return;
 
-    if (model != NULL)
-    {
-        g_return_if_fail (gtk_tree_model_get_flags (model) & GTK_TREE_MODEL_LIST_ONLY);
-
-        if (icon_bar->priv->file_column != -1)
-        {
-            file_column_type = gtk_tree_model_get_column_type (model, icon_bar->priv->file_column);
-            g_return_if_fail (file_column_type == RSTTO_TYPE_FILE);
-        }
-    }
-
     if (icon_bar->priv->model)
     {
         g_signal_handlers_disconnect_by_func (icon_bar->priv->model,
@@ -1583,72 +1447,12 @@ rstto_icon_bar_set_model (
         rstto_icon_bar_build_items (icon_bar);
     }
 
-    rstto_icon_bar_invalidate (icon_bar);
+    gtk_widget_queue_resize (GTK_WIDGET (icon_bar));
 
     g_object_notify (G_OBJECT (icon_bar), "model");
 }
 
 
-/**
- * rstto_icon_bar_get_file_column:
- * @icon_bar  : An #RsttoIconBar.
- *
- * Returns the column with file for @icon_bar.
- *
- * Returns: the file column, or -1 if it's unset.
- **/
-gint
-rstto_icon_bar_get_file_column (RsttoIconBar *icon_bar)
-{
-    g_return_val_if_fail (RSTTO_IS_ICON_BAR (icon_bar), -1);
-    return icon_bar->priv->file_column;
-}
-
-
-
-/**
- * rstto_icon_bar_set_file_column:
- * @icon_bar  : An #RsttoIconBar.
- * @column    : A column in the currently used model or -1 to
- *              use no text in @icon_bar.
- *
- * Sets the column with text for @icon_bar to be @column. The
- * text column must be of type #G_TYPE_STRING.
- **/
-void
-rstto_icon_bar_set_file_column (
-        RsttoIconBar *icon_bar,
-        gint          column)
-{
-    GType file_column_type;
-
-    g_return_if_fail (RSTTO_IS_ICON_BAR (icon_bar));
-
-    if (column == icon_bar->priv->file_column)
-        return;
-
-    if (column == -1)
-    {
-        icon_bar->priv->file_column = -1;
-    }
-    else
-    {
-        if (icon_bar->priv->model != NULL)
-        {
-            file_column_type = gtk_tree_model_get_column_type (icon_bar->priv->model, column);
-            g_return_if_fail (file_column_type == RSTTO_TYPE_FILE);
-        }
-
-        icon_bar->priv->file_column = column;
-    }
-
-    rstto_icon_bar_invalidate (icon_bar);
-
-    g_object_notify (G_OBJECT (icon_bar), "file-column");
-}
-
-
-
 /**
  * rstto_icon_bar_get_orientation:
  * @icon_bar  : An #RsttoIconBar.
@@ -1767,7 +1571,6 @@ rstto_icon_bar_get_active_iter (
         GtkTreeIter   *iter)
 {
     RsttoIconBarItem *item;
-    GtkTreePath    *path;
 
     g_return_val_if_fail (RSTTO_IS_ICON_BAR (icon_bar), FALSE);
     g_return_val_if_fail (iter != NULL, FALSE);
@@ -1776,16 +1579,7 @@ rstto_icon_bar_get_active_iter (
     if (item == NULL)
         return FALSE;
 
-    if ((gtk_tree_model_get_flags (icon_bar->priv->model) & GTK_TREE_MODEL_ITERS_PERSIST) == 0)
-    {
-        path = gtk_tree_path_new_from_indices (item->index, -1);
-        gtk_tree_model_get_iter (icon_bar->priv->model, iter, path);
-        gtk_tree_path_free (path);
-    }
-    else
-    {
-        *iter = item->iter;
-    }
+    *iter = item->iter;
 
     return TRUE;
 }
@@ -1822,19 +1616,9 @@ rstto_icon_bar_set_active_iter (
 }
 
 gint
-rstto_icon_bar_get_item_width (RsttoIconBar *icon_bar)
+rstto_icon_bar_get_n_visible_items (RsttoIconBar *icon_bar)
 {
-    return -1;
-}
-
-void
-rstto_icon_bar_set_item_width (
-        RsttoIconBar *icon_bar,
-        gint item_width)
-{
-    pango_layout_set_width (icon_bar->priv->layout,
-            item_width * PANGO_SCALE);
-    return;
+    return icon_bar->priv->n_visible_items;
 }
 
 /**
@@ -1901,8 +1685,8 @@ cb_rstto_thumbnail_size_changed (
 
     g_object_get (settings, "thumbnail-size", &icon_bar->priv->thumbnail_size, NULL);
 
-    rstto_icon_bar_invalidate (icon_bar);
     rstto_icon_bar_update_missing_icon (icon_bar);
+    gtk_widget_queue_resize (GTK_WIDGET (icon_bar));
 }
 
 static void
@@ -1929,6 +1713,7 @@ cb_rstto_icon_bar_adjustment_changed (GtkAdjustment *adjustment,
     {
         s_page_size = page_size;
         s_upper = upper;
+        icon_bar->priv->n_visible_items = 2 + page_size / icon_bar->priv->item_size;
         rstto_icon_bar_show_active (icon_bar);
     }
 }
@@ -1937,71 +1722,11 @@ static void
 rstto_icon_bar_update_missing_icon (RsttoIconBar *icon_bar)
 {
 
-    if (NULL != thumbnail_missing)
-    {
-        g_object_unref (thumbnail_missing);
-    }
-
-    switch (icon_bar->priv->thumbnail_size)
-    {
-        case THUMBNAIL_SIZE_VERY_SMALL:
-            thumbnail_missing = gtk_icon_theme_load_icon (
-                    gtk_icon_theme_get_default (),
-                    "image-missing",
-                    THUMBNAIL_SIZE_VERY_SMALL_SIZE,
-                    0,
-                    NULL);
-            break;
-        case THUMBNAIL_SIZE_SMALLER:
-            thumbnail_missing = gtk_icon_theme_load_icon (
-                    gtk_icon_theme_get_default (),
-                    "image-missing",
-                    THUMBNAIL_SIZE_SMALLER_SIZE,
-                    0,
-                    NULL);
-            break;
-        case THUMBNAIL_SIZE_SMALL:
-            thumbnail_missing = gtk_icon_theme_load_icon (
-                    gtk_icon_theme_get_default (),
-                    "image-missing",
-                    THUMBNAIL_SIZE_SMALL_SIZE,
-                    0,
-                    NULL);
-            break;
-        case THUMBNAIL_SIZE_NORMAL:
-            thumbnail_missing = gtk_icon_theme_load_icon (
-                    gtk_icon_theme_get_default (),
-                    "image-missing",
-                    THUMBNAIL_SIZE_NORMAL_SIZE,
-                    0,
-                    NULL);
-            break;
-        case THUMBNAIL_SIZE_LARGE:
-            thumbnail_missing = gtk_icon_theme_load_icon (
-                    gtk_icon_theme_get_default (),
-                    "image-missing",
-                    THUMBNAIL_SIZE_LARGE_SIZE,
-                    0,
-                    NULL);
-            break;
-        case THUMBNAIL_SIZE_LARGER:
-            thumbnail_missing = gtk_icon_theme_load_icon (
-                    gtk_icon_theme_get_default (),
-                    "image-missing",
-                    THUMBNAIL_SIZE_LARGER_SIZE,
-                    0,
-                    NULL);
-            break;
-        case THUMBNAIL_SIZE_VERY_LARGE:
-            thumbnail_missing = gtk_icon_theme_load_icon (
-                    gtk_icon_theme_get_default (),
-                    "image-missing",
-                    THUMBNAIL_SIZE_VERY_LARGE_SIZE,
-                    0,
-                    NULL);
-            break;
-        default:
-            break;
-    }
+    if (icon_bar->priv->thumbnail_missing != NULL)
+        g_object_unref (icon_bar->priv->thumbnail_missing);
 
+    icon_bar->priv->thumbnail_missing =
+        gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), "image-missing",
+                                  rstto_util_get_thumbnail_n_pixels (icon_bar->priv->thumbnail_size),
+                                  0, NULL);
 }
