@@ -53,30 +53,39 @@ static gint rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_COUNT];
 
 
 
-static void
-rstto_image_list_tree_model_init (GtkTreeModelIface *iface);
-static void
-rstto_image_list_tree_sortable_init (GtkTreeSortableIface *iface);
-
-
+/* RsttoImageList */
 static void
 rstto_image_list_finalize (GObject *object);
 
+static void
+rstto_image_list_monitor_dir (RsttoImageList *image_list,
+                              GFile *dir);
+static void
+rstto_image_list_remove_all (RsttoImageList *image_list);
+static gboolean
+rstto_image_list_set_directory_idle (gpointer data);
+static void
+rstto_image_list_set_compare_func (RsttoImageList *image_list,
+                                   GCompareDataFunc func);
+static gint
+cb_rstto_image_list_image_name_compare_func (gconstpointer a,
+                                             gconstpointer b,
+                                             gpointer user_data);
+static gint
+cb_rstto_image_list_image_type_compare_func (gconstpointer a,
+                                             gconstpointer b,
+                                             gpointer user_data);
+static gint
+cb_rstto_image_list_exif_date_compare_func (gconstpointer a,
+                                            gconstpointer b,
+                                            gpointer user_data);
+
 static void
 cb_file_monitor_changed (GFileMonitor *monitor,
                          GFile *file,
                          GFile *other_file,
                          GFileMonitorEvent event_type,
                          gpointer user_data);
-
-
-static void
-rstto_image_list_iter_finalize (GObject *object);
-
-static RsttoImageListIter *
-rstto_image_list_iter_new (RsttoImageList *nav,
-                           RsttoFile *r_file);
-
 static void
 cb_rstto_wrap_images_changed (GObject *settings,
                               GParamSpec *pspec,
@@ -85,13 +94,15 @@ static void
 cb_rstto_thumbnailer_ready (RsttoThumbnailer *thumbnailer,
                             RsttoFile *file,
                             gpointer user_data);
+
+/* RsttoImageListIter */
 static void
-rstto_image_list_monitor_dir (RsttoImageList *image_list,
-                              GFile *dir);
-static void
-rstto_image_list_remove_all (RsttoImageList *image_list);
-static gboolean
-rstto_image_list_set_directory_idle (gpointer data);
+rstto_image_list_iter_finalize (GObject *object);
+
+static RsttoImageListIter *
+rstto_image_list_iter_new (RsttoImageList *nav,
+                           RsttoFile *r_file);
+
 static gboolean
 iter_next (RsttoImageListIter *iter,
            gboolean sticky);
@@ -104,9 +115,10 @@ iter_set_position (RsttoImageListIter *iter,
                    gboolean sticky);
 
 
-/***************************************/
-/*  Begin TreeModelIface Functions     */
-/***************************************/
+/* TreeModelIface */
+static void
+rstto_image_list_tree_model_init (GtkTreeModelIface *iface);
+
 static GtkTreeModelFlags
 image_list_model_get_flags (GtkTreeModel *tree_model);
 static gint
@@ -148,47 +160,34 @@ image_list_model_iter_nth_child (GtkTreeModel *tree_model,
 static gboolean
 image_list_model_iter_next (GtkTreeModel *tree_model,
                             GtkTreeIter *iter);
-/***************************************/
-/*  End TreeModelIface Functions       */
-/***************************************/
-
-
-static gint
-cb_rstto_image_list_image_name_compare_func (gconstpointer a,
-                                             gconstpointer b);
-static gint
-cb_rstto_image_list_image_type_compare_func (gconstpointer a,
-                                             gconstpointer b);
-static gint
-cb_rstto_image_list_exif_date_compare_func (gconstpointer a,
-                                            gconstpointer b);
 
 
 
 struct _RsttoImageListIterPrivate
 {
     RsttoImageList *image_list;
-    RsttoFile      *r_file;
-
-    /* This is set if the iter-position is chosen by the user */
-    gboolean        sticky;
+    RsttoFile *r_file;
+    GList *link;
+    gint index;
+    gboolean sticky;
 };
 
 struct _RsttoImageListPrivate
 {
-    gint           stamp;
-    GFileMonitor  *dir_monitor;
-    gboolean       is_busy;
     RsttoSettings *settings;
     RsttoThumbnailer *thumbnailer;
 
-    GList        *image_monitors;
-    GList        *images;
+    GFileMonitor *dir_monitor;
+    GList *image_monitors;
 
-    GSList       *iterators;
-    GCompareFunc  cb_rstto_image_list_compare_func;
+    GQueue *images;
+    GSList *iterators;
+    GCompareDataFunc cb_rstto_image_list_compare_func;
 
-    gboolean      wrap_images;
+    gboolean wrap_images;
+    gboolean is_busy;
+
+    gint stamp;
 };
 
 
@@ -196,9 +195,7 @@ struct _RsttoImageListPrivate
 G_DEFINE_TYPE_WITH_CODE (RsttoImageList, rstto_image_list, G_TYPE_OBJECT,
                          G_ADD_PRIVATE (RsttoImageList)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL,
-                                                rstto_image_list_tree_model_init)
-                         G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_SORTABLE,
-                                                rstto_image_list_tree_sortable_init))
+                                                rstto_image_list_tree_model_init))
 
 G_DEFINE_TYPE_WITH_PRIVATE (RsttoImageListIter, rstto_image_list_iter, G_TYPE_OBJECT)
 
@@ -221,18 +218,6 @@ rstto_image_list_tree_model_init (GtkTreeModelIface *iface)
     iface->iter_next       = image_list_model_iter_next;
 }
 
-static void
-rstto_image_list_tree_sortable_init (GtkTreeSortableIface *iface)
-{
-#if 0
-    iface->get_sort_column_id    = sq_archive_store_get_sort_column_id;
-    iface->set_sort_column_id    = sq_archive_store_set_sort_column_id;
-    iface->set_sort_func         = sq_archive_store_set_sort_func;            /*NOT SUPPORTED*/
-    iface->set_default_sort_func = sq_archive_store_set_default_sort_func;    /*NOT SUPPORTED*/
-    iface->has_default_sort_func = sq_archive_store_has_default_sort_func;
-#endif
-}
-
 static void
 rstto_image_list_init (RsttoImageList *image_list)
 {
@@ -241,6 +226,7 @@ rstto_image_list_init (RsttoImageList *image_list)
     image_list->priv->is_busy = FALSE;
     image_list->priv->settings = rstto_settings_new ();
     image_list->priv->thumbnailer = rstto_thumbnailer_new ();
+    image_list->priv->images = g_queue_new ();
 
     image_list->priv->cb_rstto_image_list_compare_func = cb_rstto_image_list_image_name_compare_func;
 
@@ -313,7 +299,7 @@ rstto_image_list_finalize (GObject *object)
 
         if (image_list->priv->images)
         {
-            g_list_free_full (image_list->priv->images, g_object_unref);
+            g_queue_free_full (image_list->priv->images, g_object_unref);
             image_list->priv->images = NULL;
         }
     }
@@ -345,7 +331,7 @@ rstto_image_list_add_file (RsttoImageList *image_list,
     g_return_val_if_fail (RSTTO_IS_FILE (r_file), FALSE);
 
     /* file already added */
-    if (g_list_find (image_list->priv->images, r_file))
+    if (g_queue_find (image_list->priv->images, r_file))
         return TRUE;
 
     /* unsupported mime type */
@@ -355,8 +341,8 @@ rstto_image_list_add_file (RsttoImageList *image_list,
         return FALSE;
     }
 
-    image_list->priv->images = g_list_insert_sorted (image_list->priv->images, g_object_ref (r_file),
-                                                     image_list->priv->cb_rstto_image_list_compare_func);
+    g_queue_insert_sorted (image_list->priv->images, g_object_ref (r_file),
+                           image_list->priv->cb_rstto_image_list_compare_func, NULL);
 
     if (image_list->priv->dir_monitor == NULL)
     {
@@ -366,7 +352,7 @@ rstto_image_list_add_file (RsttoImageList *image_list,
     }
 
     path = gtk_tree_path_new ();
-    idx = g_list_index (image_list->priv->images, r_file);
+    idx = g_queue_index (image_list->priv->images, r_file);
     gtk_tree_path_append_index (path, idx);
     t_iter.stamp = image_list->priv->stamp;
     t_iter.user_data = r_file;
@@ -385,7 +371,13 @@ rstto_image_list_add_file (RsttoImageList *image_list,
 gint
 rstto_image_list_get_n_images (RsttoImageList *image_list)
 {
-    return g_list_length (image_list->priv->images);
+    return image_list->priv->images->length;
+}
+
+gboolean
+rstto_image_list_is_empty (RsttoImageList *image_list)
+{
+    return g_queue_is_empty (image_list->priv->images);
 }
 
 /**
@@ -402,9 +394,9 @@ rstto_image_list_get_iter (RsttoImageList *image_list)
     RsttoFile *r_file = NULL;
     RsttoImageListIter *iter = NULL;
 
-    if (image_list->priv->images)
+    if (! g_queue_is_empty (image_list->priv->images))
     {
-        r_file = image_list->priv->images->data;
+        r_file = image_list->priv->images->head->data;
     }
 
     iter = rstto_image_list_iter_new (image_list, r_file);
@@ -416,75 +408,49 @@ rstto_image_list_get_iter (RsttoImageList *image_list)
 
 
 void
-rstto_image_list_remove_file (
-        RsttoImageList *image_list,
-        RsttoFile *r_file)
+rstto_image_list_remove_file (RsttoImageList *image_list,
+                              RsttoFile *r_file)
 {
-    GSList *iter = NULL;
-    RsttoFile *r_file_a = NULL;
-    GtkTreePath *path_ = NULL;
-    gint index_ = g_list_index (image_list->priv->images, r_file);
-    gint n_images = rstto_image_list_get_n_images (image_list);
+    RsttoImageListIter *r_iter;
+    GSList *iter;
+    GtkTreePath *path_;
+    gint index_;
 
-    if (index_ != -1)
+    index_ = g_queue_index (image_list->priv->images, r_file);
+    if (index_ == -1)
+        return;
+
+    for (iter = image_list->priv->iterators; iter != NULL; iter = iter->next)
     {
+        r_iter = iter->data;
+        if (r_iter->priv->r_file != r_file)
+            continue;
 
-        iter = image_list->priv->iterators;
-        while (iter)
+        if (r_iter->priv->link == image_list->priv->images->tail)
+            iter_previous (r_iter, r_iter->priv->sticky);
+        else
+            iter_next (r_iter, r_iter->priv->sticky);
+
+        /* if the image is still the same, it's a single item list,
+         * and we should force the image in this iter to NULL */
+        if (r_iter->priv->r_file == r_file)
         {
-            if (rstto_file_equal (rstto_image_list_iter_get_file (iter->data), r_file))
-            {
-                if (rstto_image_list_iter_get_position (iter->data) == n_images - 1)
-                {
-                    iter_previous (iter->data, FALSE);
-                }
-                else
-                {
-                    iter_next (iter->data, FALSE);
-                }
-                /* If the image is still the same,
-                 * it's a single item list,
-                 * and we should force the image in this iter to NULL
-                 */
-                if (rstto_file_equal (
-                        rstto_image_list_iter_get_file (iter->data),
-                        r_file))
-                {
-
-                    image_list->priv->images = g_list_remove (image_list->priv->images, r_file);
-                    RSTTO_IMAGE_LIST_ITER (iter->data)->priv->r_file = NULL;
-                    g_signal_emit (iter->data,
-                            rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_CHANGED],
-                            0, NULL);
-                }
-            }
-            iter = g_slist_next (iter);
+            r_iter->priv->r_file = NULL;
+            r_iter->priv->link = NULL;
+            r_iter->priv->index = -1;
+            g_signal_emit (r_iter,
+                           rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_CHANGED],
+                           0, NULL);
         }
-
-        image_list->priv->images = g_list_remove (image_list->priv->images, r_file);
-
-        path_ = gtk_tree_path_new ();
-        gtk_tree_path_append_index (path_, index_);
-
-        gtk_tree_model_row_deleted (GTK_TREE_MODEL (image_list), path_);
-
-        iter = image_list->priv->iterators;
-        while (iter)
-        {
-            r_file_a = rstto_image_list_iter_get_file (iter->data);
-            if (NULL != r_file_a)
-            {
-                if (rstto_file_equal (r_file_a, r_file))
-                {
-                    iter_next (iter->data, FALSE);
-                }
-            }
-            iter = g_slist_next (iter);
-        }
-
-        g_object_unref (r_file);
-        gtk_tree_path_free (path_);
     }
+
+    g_queue_remove (image_list->priv->images, r_file);
+    g_object_unref (r_file);
+
+    path_ = gtk_tree_path_new ();
+    gtk_tree_path_append_index (path_, index_);
+    gtk_tree_model_row_deleted (GTK_TREE_MODEL (image_list), path_);
+    gtk_tree_path_free (path_);
 }
 
 static void
@@ -495,8 +461,8 @@ rstto_image_list_remove_all (RsttoImageList *image_list)
     g_list_free_full (image_list->priv->image_monitors, g_object_unref);
     image_list->priv->image_monitors = NULL;
 
-    g_list_free_full (image_list->priv->images, g_object_unref);
-    image_list->priv->images = NULL;
+    g_queue_free_full (image_list->priv->images, g_object_unref);
+    image_list->priv->images = g_queue_new ();
 
     for (iter = image_list->priv->iterators; iter != NULL; iter = iter->next)
         iter_set_position (iter->data, -1, FALSE);
@@ -546,7 +512,7 @@ rstto_image_list_set_directory_finish_idle (gpointer data)
             /* load the first valid file, if needed */
             r_file = rstto_file_new (file);
             g_object_unref (file);
-            if (loaded_file == NULL && image_list->priv->images == NULL)
+            if (loaded_file == NULL && g_queue_is_empty (image_list->priv->images))
             {
                 if (! rstto_image_list_add_file (image_list, r_file, NULL))
                 {
@@ -567,7 +533,7 @@ rstto_image_list_set_directory_finish_idle (gpointer data)
             /* no additional filtering here, it will be done gradually when validating files,
              * and the mime type will be refined at this time as well */
             rstto_file_set_content_type (r_file, filter_info.mime_type);
-            image_list->priv->images = g_list_prepend (image_list->priv->images, r_file);
+            g_queue_push_head (image_list->priv->images, r_file);
         }
     }
 
@@ -578,7 +544,7 @@ rstto_image_list_set_directory_finish_idle (gpointer data)
     {
         /* we're done, inform the others */
         image_list->priv->is_busy = FALSE;
-        if (image_list->priv->images != NULL)
+        if (! g_queue_is_empty (image_list->priv->images))
             rstto_image_list_set_compare_func (image_list,
                                                image_list->priv->cb_rstto_image_list_compare_func);
         else
@@ -613,7 +579,7 @@ rstto_image_list_set_directory_finish (GObject *source_object,
     {
         /* we're done, inform the others */
         image_list->priv->is_busy = FALSE;
-        if (image_list->priv->images != NULL)
+        if (! g_queue_is_empty (image_list->priv->images))
         {
             rstto_image_list_set_compare_func (image_list,
                                                image_list->priv->cb_rstto_image_list_compare_func);
@@ -755,7 +721,7 @@ cb_file_monitor_changed (
     static RsttoFile *s_r_file = NULL;
 
     /* see if the file is already open */
-    for (list = image_list->priv->images; list != NULL; list = list->next)
+    for (list = image_list->priv->images->head; list != NULL; list = list->next)
         if (g_file_equal (file, rstto_file_get_file (list->data)))
         {
             r_file = g_object_ref (list->data);
@@ -787,15 +753,29 @@ cb_file_monitor_changed (
             s_r_file = g_object_ref (r_file);
             break;
         case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
+            /* a file was created */
             if (s_r_file != NULL)
             {
                 rstto_image_list_add_file (image_list, s_r_file, NULL);
                 g_object_unref (s_r_file);
                 s_r_file = NULL;
             }
+            /* a file has changed */
+            else
+            {
+                rstto_file_changed (r_file);
+                if (image_list->priv->cb_rstto_image_list_compare_func ==
+                        cb_rstto_image_list_image_type_compare_func
+                    || image_list->priv->cb_rstto_image_list_compare_func ==
+                        cb_rstto_image_list_exif_date_compare_func)
+                {
+                    rstto_image_list_set_compare_func (image_list,
+                        image_list->priv->cb_rstto_image_list_compare_func);
+                }
+            }
             break;
         case G_FILE_MONITOR_EVENT_CHANGED:
-            rstto_file_changed (r_file);
+            /* wait for DONE_HINT to update the image, so that we can get its mime type */
             break;
         default:
             break;
@@ -853,6 +833,8 @@ rstto_image_list_iter_finalize (GObject *object)
     if (iter->priv->r_file)
     {
         iter->priv->r_file = NULL;
+        iter->priv->link = NULL;
+        iter->priv->index = -1;
     }
 
     if (iter->priv->image_list)
@@ -872,8 +854,10 @@ rstto_image_list_iter_new (
     RsttoImageListIter *iter;
 
     iter = g_object_new (RSTTO_TYPE_IMAGE_LIST_ITER, NULL);
-    iter->priv->r_file = r_file;
     iter->priv->image_list = nav;
+    iter->priv->r_file = r_file;
+    iter->priv->index = g_queue_index (nav->priv->images, r_file);
+    iter->priv->link = g_queue_peek_nth_link (nav->priv->images, iter->priv->index);
 
     return iter;
 }
@@ -883,38 +867,33 @@ rstto_image_list_iter_find_file (
         RsttoImageListIter *iter,
         RsttoFile *r_file)
 {
-    gint pos = g_list_index (iter->priv->image_list->priv->images, r_file);
+    RsttoImageList *image_list = iter->priv->image_list;
+    gint index;
 
-    if (pos > -1)
-    {
-        g_signal_emit (iter,
-                       rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_PREPARE_CHANGE],
-                       0, NULL);
+    index = g_queue_index (image_list->priv->images, r_file);
+    if (index == -1)
+        return FALSE;
 
-        if (iter->priv->r_file)
-        {
-            iter->priv->r_file = NULL;
-        }
-        iter->priv->r_file = r_file;
-        iter->priv->sticky = TRUE;
+    g_signal_emit (iter,
+                   rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_PREPARE_CHANGE],
+                   0, NULL);
 
-        g_signal_emit (iter,
-                       rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_CHANGED],
-                       0, NULL);
+    iter->priv->r_file = r_file;
+    iter->priv->index = index;
+    iter->priv->link = g_queue_peek_nth_link (image_list->priv->images, iter->priv->index);
+    iter->priv->sticky = TRUE;
 
-        return TRUE;
-    }
-    return FALSE;
+    g_signal_emit (iter,
+                   rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_CHANGED],
+                   0, NULL);
+
+    return TRUE;
 }
 
 gint
 rstto_image_list_iter_get_position (RsttoImageListIter *iter)
 {
-    if (NULL == iter->priv->r_file)
-    {
-        return -1;
-    }
-    return g_list_index (iter->priv->image_list->priv->images, iter->priv->r_file);
+    return iter->priv->index;
 }
 
 RsttoFile *
@@ -929,19 +908,24 @@ iter_set_position (
         gint pos,
         gboolean sticky)
 {
+    if (iter->priv->index == pos)
+        return;
+
     g_signal_emit (iter,
                    rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_PREPARE_CHANGE],
                    0, NULL);
 
     iter->priv->sticky = sticky;
-    if (iter->priv->r_file)
+    iter->priv->link = g_queue_peek_nth_link (iter->priv->image_list->priv->images, pos);
+    if (iter->priv->link == NULL)
     {
         iter->priv->r_file = NULL;
+        iter->priv->index = -1;
     }
-
-    if (pos >= 0)
+    else
     {
-        iter->priv->r_file = g_list_nth_data (iter->priv->image_list->priv->images, pos);
+        iter->priv->r_file = iter->priv->link->data;
+        iter->priv->index = pos;
     }
 
     g_signal_emit (iter,
@@ -962,64 +946,38 @@ iter_next (
         RsttoImageListIter *iter,
         gboolean sticky)
 {
-    GList *position = NULL;
     RsttoImageList *image_list = iter->priv->image_list;
-    RsttoFile *r_file = iter->priv->r_file;
     gboolean ret_val = FALSE;
 
+    if (iter->priv->link == NULL)
+        return FALSE;
+
     g_signal_emit (iter,
                    rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_PREPARE_CHANGE],
                    0, NULL);
 
-    if (r_file)
+    if (iter->priv->link->next != NULL || image_list->priv->wrap_images)
     {
-        position = g_list_find (iter->priv->image_list->priv->images, r_file);
-        iter->priv->r_file = NULL;
-    }
+        if (iter->priv->link->next != NULL)
+        {
+            iter->priv->link = iter->priv->link->next;
+            iter->priv->index++;
+        }
+        else
+        {
+            iter->priv->link = image_list->priv->images->head;
+            iter->priv->index = 0;
+        }
 
-    iter->priv->sticky = sticky;
-
-    position = g_list_next (position);
-    if (position)
-    {
-        iter->priv->r_file = position->data;
-
-        /* We could move forward, set ret_val to TRUE */
+        iter->priv->r_file = iter->priv->link->data;
+        iter->priv->sticky = sticky;
         ret_val = TRUE;
     }
-    else
-    {
 
-        if (image_list->priv->wrap_images)
-        {
-            position = g_list_first (iter->priv->image_list->priv->images);
-
-            /* We could move forward, wrapped back to the start of the
-             * list, set ret_val to TRUE
-             */
-            ret_val = TRUE;
-        }
-        else
-        {
-            position = g_list_last (iter->priv->image_list->priv->images);
-        }
-
-        if (position)
-        {
-            iter->priv->r_file = position->data;
-        }
-        else
-        {
-            iter->priv->r_file = NULL;
-        }
-    }
-
-    if (r_file != iter->priv->r_file)
-    {
+    if (ret_val == TRUE)
         g_signal_emit (iter,
                        rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_CHANGED],
                        0, NULL);
-    }
 
     return ret_val;
 }
@@ -1035,22 +993,10 @@ rstto_image_list_iter_has_next (RsttoImageListIter *iter)
 {
     RsttoImageList *image_list = iter->priv->image_list;
 
-    if (image_list->priv->wrap_images)
-    {
-        return TRUE;
-    }
-    else
-    {
-        if (rstto_image_list_iter_get_position (iter) ==
-            (rstto_image_list_get_n_images (image_list) - 1))
-        {
-            return FALSE;
-        }
-        else
-        {
-            return TRUE;
-        }
-    }
+    return iter->priv->link != NULL && (
+                image_list->priv->wrap_images
+                || iter->priv->link != image_list->priv->images->tail
+            );
 }
 
 static gboolean
@@ -1058,60 +1004,40 @@ iter_previous (
         RsttoImageListIter *iter,
         gboolean sticky)
 {
-    GList *position = NULL;
     RsttoImageList *image_list = iter->priv->image_list;
-    RsttoFile *r_file = iter->priv->r_file;
     gboolean ret_val = FALSE;
 
+    if (iter->priv->link == NULL)
+        return FALSE;
+
     g_signal_emit (iter,
                    rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_PREPARE_CHANGE],
                    0, NULL);
 
-    if (iter->priv->r_file)
+    if (iter->priv->link->prev != NULL || image_list->priv->wrap_images)
     {
-        position = g_list_find (iter->priv->image_list->priv->images, iter->priv->r_file);
-        iter->priv->r_file = NULL;
-    }
+        if (iter->priv->link->prev != NULL)
+        {
+            iter->priv->link = iter->priv->link->prev;
+            iter->priv->index--;
+        }
+        else
+        {
+            iter->priv->link = image_list->priv->images->tail;
+            iter->priv->index = image_list->priv->images->length - 1;
+        }
 
-    iter->priv->sticky = sticky;
-
-    position = g_list_previous (position);
-    if (position)
-    {
-        iter->priv->r_file = position->data;
+        iter->priv->r_file = iter->priv->link->data;
+        iter->priv->sticky = sticky;
         ret_val = TRUE;
     }
-    else
-    {
-        if (image_list->priv->wrap_images)
-        {
-            position = g_list_last (iter->priv->image_list->priv->images);
-            ret_val = TRUE;
-        }
-        else
-        {
-            position = g_list_first (iter->priv->image_list->priv->images);
-        }
 
-        if (position)
-        {
-            iter->priv->r_file = position->data;
-        }
-        else
-        {
-            iter->priv->r_file = NULL;
-        }
-    }
-
-    if (r_file != iter->priv->r_file)
-    {
+    if (ret_val == TRUE)
         g_signal_emit (iter,
                        rstto_image_list_iter_signals[RSTTO_IMAGE_LIST_ITER_SIGNAL_CHANGED],
                        0, NULL);
-    }
 
     return ret_val;
-
 }
 gboolean
 rstto_image_list_iter_previous (RsttoImageListIter *iter)
@@ -1125,38 +1051,28 @@ rstto_image_list_iter_has_previous (RsttoImageListIter *iter)
 {
     RsttoImageList *image_list = iter->priv->image_list;
 
-    if (image_list->priv->wrap_images)
-    {
-        return TRUE;
-    }
-    else
-    {
-        if (rstto_image_list_iter_get_position (iter) == 0)
-        {
-            return FALSE;
-        }
-        else
-        {
-            return TRUE;
-        }
-    }
+    return iter->priv->link != NULL && (
+                image_list->priv->wrap_images
+                || iter->priv->link != image_list->priv->images->head
+            );
 }
 
 
 RsttoImageListIter *
 rstto_image_list_iter_clone (RsttoImageListIter *iter)
 {
-    return rstto_image_list_iter_new (iter->priv->image_list, iter->priv->r_file);
+    RsttoImageListIter *new_iter;
+
+    new_iter = rstto_image_list_iter_new (iter->priv->image_list, NULL);
+    new_iter->priv->r_file = iter->priv->r_file;
+    new_iter->priv->link = iter->priv->link;
+    new_iter->priv->index = iter->priv->index;
+
+    return new_iter;
 }
 
-GCompareFunc
-rstto_image_list_get_compare_func (RsttoImageList *image_list)
-{
-    return image_list->priv->cb_rstto_image_list_compare_func;
-}
-
-void
-rstto_image_list_set_compare_func (RsttoImageList *image_list, GCompareFunc func)
+static void
+rstto_image_list_set_compare_func (RsttoImageList *image_list, GCompareDataFunc func)
 {
     RsttoFile *files[g_slist_length (image_list->priv->iterators)];
     GSList *iter;
@@ -1166,9 +1082,9 @@ rstto_image_list_set_compare_func (RsttoImageList *image_list, GCompareFunc func
 
     /* store iter files before sorting */
     for (iter = image_list->priv->iterators, n = 0; iter != NULL; iter = iter->next, n++)
-        files[n] = rstto_image_list_iter_get_file (iter->data);
+        files[n] = RSTTO_IMAGE_LIST_ITER (iter->data)->priv->r_file;
 
-    image_list->priv->images = g_list_sort (image_list->priv->images, func);
+    g_queue_sort (image_list->priv->images, func, NULL);
 
     /* reposition iters on their file */
     for (iter = image_list->priv->iterators, n = 0; iter != NULL; iter = iter->next, n++)
@@ -1209,7 +1125,8 @@ rstto_image_list_set_sort_by_date (RsttoImageList *image_list)
  */
 static gint
 cb_rstto_image_list_image_name_compare_func (gconstpointer a,
-                                             gconstpointer b)
+                                             gconstpointer b,
+                                             gpointer user_data)
 {
     const gchar *a_collate_key = rstto_file_get_collate_key ((RsttoFile *) a);
     const gchar *b_collate_key = rstto_file_get_collate_key ((RsttoFile *) b);
@@ -1227,7 +1144,8 @@ cb_rstto_image_list_image_name_compare_func (gconstpointer a,
  */
 static gint
 cb_rstto_image_list_image_type_compare_func (gconstpointer a,
-                                             gconstpointer b)
+                                             gconstpointer b,
+                                             gpointer user_data)
 {
     const gchar *a_content_type = rstto_file_get_content_type ((RsttoFile *) a);
     const gchar *b_content_type = rstto_file_get_content_type ((RsttoFile *) b);
@@ -1246,7 +1164,8 @@ cb_rstto_image_list_image_type_compare_func (gconstpointer a,
  */
 static gint
 cb_rstto_image_list_exif_date_compare_func (gconstpointer a,
-                                            gconstpointer b)
+                                            gconstpointer b,
+                                            gpointer user_data)
 {
     guint64 a_t = rstto_file_get_modified_time ((RsttoFile *) a);
     guint64 b_t = rstto_file_get_modified_time ((RsttoFile *) b);
@@ -1293,7 +1212,7 @@ image_list_model_get_n_columns (GtkTreeModel *tree_model)
 {
     g_return_val_if_fail (RSTTO_IS_IMAGE_LIST (tree_model), 0);
 
-    return 2;
+    return 1;
 }
 
 static GType
@@ -1305,14 +1224,11 @@ image_list_model_get_column_type (
 
     switch (index_)
     {
-        case 0: /* file */
+        case 0:
             return RSTTO_TYPE_FILE;
-        case 1:
-            return GDK_TYPE_PIXBUF;
-            break;
         default:
+            g_warn_if_reached ();
             return G_TYPE_INVALID;
-            break;
     }
 }
 
@@ -1342,7 +1258,7 @@ image_list_model_get_iter (
 
     if (index_ >= 0)
     {
-        file = g_list_nth_data (image_list->priv->images, index_);
+        file = g_queue_peek_nth (image_list->priv->images, index_);
     }
 
     if (NULL == file)
@@ -1394,7 +1310,7 @@ image_list_model_iter_children (
 
     image_list = RSTTO_IMAGE_LIST (tree_model);
 
-    file = g_list_nth_data (image_list->priv->images, 0);
+    file = image_list->priv->images->head->data;
 
     if (NULL == file)
     {
@@ -1459,10 +1375,10 @@ image_list_model_iter_next (
 
     pos = GPOINTER_TO_INT (iter->user_data3);
     if (pos == 0)
-        s_list = image_list->priv->images;
+        s_list = image_list->priv->images->head;
     /* unlikely, and in any case should happen only once at initialization */
     else if (pos != s_pos)
-        s_list = g_list_nth (image_list->priv->images, pos);
+        s_list = g_queue_peek_nth_link (image_list->priv->images, pos);
 
     if (s_list == NULL || s_list->next == NULL)
         return FALSE;
@@ -1490,8 +1406,11 @@ image_list_model_get_value (
     {
         case 0:
             g_value_init (value, RSTTO_TYPE_FILE);
-            g_value_set_object (value, file);
+            g_value_take_object (value, file);
             break;
+        default:
+            g_warn_if_reached ();
+            return;
     }
 }
 
@@ -1506,7 +1425,7 @@ cb_rstto_thumbnailer_ready (
     GtkTreeIter iter;
     gint index_;
 
-    index_ = g_list_index (image_list->priv->images, file);
+    index_ = g_queue_index (image_list->priv->images, file);
     if (index_ >= 0)
     {
         path_ = gtk_tree_path_new ();
