@@ -62,8 +62,6 @@ typedef enum
     RSTTO_IMAGE_VIEWER_MOTION_STATE_MOVE
 } RsttoImageViewerMotionState;
 
-static GdkScreen *default_screen = NULL;
-
 typedef struct _RsttoImageViewerTransaction RsttoImageViewerTransaction;
 
 
@@ -215,6 +213,15 @@ struct _RsttoImageViewerTransaction
     gdouble           image_scale;
     gdouble           scale;
 
+    /*
+     * GDK uses windowing system specific APIs to retrieve this, and X11 requires
+     * special processing to be used in multi-threaded context. So this must be
+     * retrieved before we start asynchronous image loading, so that we don't have
+     * to distinguish between windowing systems, or use X11-specific APIs in our code.
+     */
+    gint              monitor_width;
+    gint              monitor_height;
+
     /* File I/O data */
     /*****************/
     guchar           *buffer;
@@ -318,11 +325,6 @@ rstto_image_viewer_init (RsttoImageViewer *viewer)
     GSList *list, *li;
     gchar **strv;
 
-    if (default_screen == NULL)
-    {
-        default_screen = gdk_screen_get_default ();
-    }
-
     viewer->priv = rstto_image_viewer_get_instance_private (viewer);
     viewer->priv->settings = rstto_settings_new ();
     viewer->priv->pixbuf.pattern = NULL;
@@ -755,11 +757,11 @@ set_device_scale (GObject *window,
     RsttoImageViewer *viewer = data;
     gint scale;
 
+    g_object_get (window, "device-scale", &scale, NULL);
+    viewer->priv->device_scale = scale;
+
     if (viewer->priv->image_width > 0)
     {
-        g_object_get (window, "device-scale", &scale, NULL);
-        viewer->priv->device_scale = scale;
-
         /* do not scale the image with the rest of the window */
         viewer->priv->image_width = viewer->priv->original_image_width / scale;
         viewer->priv->image_height = viewer->priv->original_image_height / scale;
@@ -1262,7 +1264,7 @@ rstto_image_viewer_set_file (RsttoImageViewer *viewer,
             /*
              * If the old, and new file are equal, do nothing.
              */
-            if (!rstto_file_equal (viewer->priv->file, file))
+            if (viewer->priv->file != file)
             {
                 /*
                  * This will first need to return to the 'main' loop before it cleans up after itself.
@@ -1391,6 +1393,18 @@ rstto_image_viewer_load_image (RsttoImageViewer *viewer, RsttoFile *file, gdoubl
     transaction->scale = scale;
     transaction->loader_closed_id = 0;
 
+    if (viewer->priv->limit_quality)
+    {
+        GdkMonitor *monitor;
+        GdkRectangle rect;
+
+        monitor = gdk_display_get_monitor_at_window (gdk_display_get_default (),
+                                                     gtk_widget_get_window (GTK_WIDGET (viewer)));
+        gdk_monitor_get_geometry (monitor, &rect);
+        transaction->monitor_width = rect.width * viewer->priv->device_scale;
+        transaction->monitor_height = rect.height * viewer->priv->device_scale;
+    }
+
     g_signal_connect (transaction->loader, "size-prepared", G_CALLBACK (cb_rstto_image_loader_size_prepared), transaction);
     g_signal_connect (transaction->loader, "closed", G_CALLBACK (cb_rstto_image_loader_closed), transaction);
 
@@ -1678,8 +1692,6 @@ cb_rstto_image_loader_image_ready (GdkPixbufLoader *loader, RsttoImageViewerTran
 static void
 cb_rstto_image_loader_size_prepared (GdkPixbufLoader *loader, gint width, gint height, RsttoImageViewerTransaction *transaction)
 {
-    gboolean limit_quality = transaction->viewer->priv->limit_quality;
-
     /*
      * By default, the image-size won't be limited to screen-size (since it's smaller)
      * or, because we don't want to reduce it.
@@ -1690,28 +1702,19 @@ cb_rstto_image_loader_size_prepared (GdkPixbufLoader *loader, gint width, gint h
     transaction->image_width = width;
     transaction->image_height = height;
 
-    if (limit_quality)
+    if (transaction->viewer->priv->limit_quality)
     {
-        GdkMonitor *monitor = gdk_display_get_monitor_at_window (
-                gdk_screen_get_display (default_screen),
-                gtk_widget_get_window (GTK_WIDGET (transaction->viewer)));
-        gdouble s_width, s_height;
-        GdkRectangle monitor_geometry;
-
-        gdk_monitor_get_geometry (monitor, &monitor_geometry);
-        s_width = monitor_geometry.width;
-        s_height = monitor_geometry.height;
-
         /*
          * Set the maximum size of the loaded image to the screen-size.
          * TODO: Add some 'smart-stuff' here
          */
-        if (s_width < width || s_height < height)
+        if (transaction->monitor_width < width || transaction->monitor_height < height)
         {
+            gdouble s_width = transaction->monitor_width, s_height = transaction->monitor_height;
+
             /*
              * The image is loaded at the screen_size, calculate how this fits best.
              *  scale = MIN (width / screen_width, height / screen_height)
-             *
              */
             if (width / s_width < height / s_height)
             {
@@ -1724,14 +1727,6 @@ cb_rstto_image_loader_size_prepared (GdkPixbufLoader *loader, gint width, gint h
                 gdk_pixbuf_loader_set_size (loader, s_height * width / height, s_height);
             }
         }
-        else
-        {
-            /*
-             * Image-size won't be limited to screen-size (since it's smaller)
-             * Set the image_scale to 1.0 (100%)
-             */
-            transaction->image_scale = 1.0;
-        }
     }
 }
 
@@ -1783,9 +1778,9 @@ cb_rstto_image_loader_closed_idle (gpointer data)
                          gtk_adjustment_get_value (viewer->priv->hadjustment),
                          gtk_adjustment_get_value (viewer->priv->vadjustment));
         gdk_window_invalidate_rect (gtk_widget_get_window (widget), NULL, FALSE);
-    }
 
-    g_signal_emit_by_name (transaction->viewer, "size-ready");
+        g_signal_emit_by_name (transaction->viewer, "size-ready");
+    }
 
     transaction->loader_closed_id = 0;
 
@@ -2308,10 +2303,6 @@ rstto_image_viewer_is_busy (RsttoImageViewer *viewer)
 static void
 cb_rstto_image_viewer_file_changed (RsttoFile *r_file, RsttoImageViewer *viewer)
 {
-    rstto_image_viewer_load_image (
-            viewer,
-            r_file,
-            viewer->priv->scale);
-
+    rstto_image_viewer_load_image (viewer, r_file, viewer->priv->auto_scale);
     g_signal_emit_by_name (viewer, "status-changed");
 }
