@@ -25,6 +25,16 @@
 #ifndef __RISTRETTO_UTIL_H__
 #define __RISTRETTO_UTIL_H__
 
+/* disable debug checks in release mode (before including GLib/GTK headers) */
+#ifdef NDEBUG
+#ifndef G_DISABLE_CHECKS
+#define G_DISABLE_CHECKS
+#endif
+#ifndef G_DISABLE_ASSERT
+#define G_DISABLE_ASSERT
+#endif
+#endif
+
 /* our config header */
 #include <config.h>
 
@@ -36,6 +46,7 @@
 
 /* internal headers */
 #include "settings.h"
+#include "file.h"
 
 G_BEGIN_DECLS
 
@@ -82,6 +93,18 @@ void
 rstto_util_dialog_error (const gchar *message,
                          GError *error);
 
+RsttoThumbnailFlavor
+rstto_util_get_thumbnail_flavor (RsttoThumbnailSize size);
+
+RsttoThumbnailSize
+rstto_util_get_thumbnail_size (RsttoThumbnailFlavor flavor);
+
+const gchar *
+rstto_util_get_thumbnail_flavor_name (RsttoThumbnailFlavor flavor);
+
+guint
+rstto_util_get_thumbnail_n_pixels (RsttoThumbnailSize size);
+
 G_END_DECLS
 
 #endif /* __RSTTO_UTIL_H__ */
